/*                                                                -*- mode: c; c-basic-offset: 8;  -*-
 * Copyright (c) Members of the EGEE Collaboration. 2004.
 * See http://www.eu-egee.org/partners/ for details on the copyright holders.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/*
 ============================================================================
 Implementation of Glue1.3 for gfal
 ============================================================================
 */



#ifndef __GNUC__
#define __attribute__(x)
#endif

#include <stdio.h>
#include "Common.h"
#include "Definitions.h"
#include "ServiceDiscoveryIfce.h"


static char *glue1_base = "o=grid";
extern char *vo_global;
extern char **fqan_global;
extern int fqan_size_global;

/*  get from the BDII the CE Accesspoint for a given SE
  */
int g1_sd_get_ce_ap(const char *host, char **ce_ap, char *errbuf, int errbufsz) {
    LDAPMessage *entry = NULL;
    SDException exception;
    LDAPMessage *reply = NULL;
    struct berval **value = NULL;
    char *query = NULL;
    int sav_errno = 0;
    *ce_ap = NULL;
    int rc = 0;
    LDAP *ld = NULL;

    if (!host) {
        SD_setException(&exception, SDStatus_FAILURE, "[BDII][g1_sd_get_ce_ap]: Hostname is not specified", errbuf, errbufsz);
        return (-1);
    }

    query = g_strdup_printf(QUERY_CESE_BIND, host);


    if ((ld = get_connection(&exception, errbuf, errbufsz)) == NULL) {
        SD_setException(&exception, SDStatus_FAILURE, "[BDII][g1_sd_get_ce_ap]: Cannot connect to BDII", errbuf, errbufsz);
        if (query)
            g_free(query);
        return (-1);
    }

       rc = connect_search_ldap(glue1_base, query, ce_ap_attrs, ld, &exception, &reply, errbuf, errbufsz);
        if (rc != LDAP_SUCCESS) {
           if (reply)
                ldap_msgfree(reply);
            if (query)
               g_free(query);
	    if (ld)
        	close_connection(ld);	
            sav_errno = errno;
	    rc = -1;
	    goto out;
            
        }



        if ((1 > ldap_count_entries(ld, reply))) {
            if (reply)
                ldap_msgfree(reply);
            if (query)
               g_free(query);
	    if (ld)
        	close_connection(ld);	       
            return (-1);
        }


        for (entry = ldap_first_entry(ld, reply); entry != NULL && rc == 0; entry
                = ldap_next_entry(ld, entry)) {


        if ((entry = ldap_first_entry(ld, reply)) && (value
                = ldap_get_values_len(ld, entry, ATTR_CE_AP_ATNM))) {
            if (value && value[0]->bv_val) {
                if ((*ce_ap = strdup(value[0]->bv_val)) == NULL) {
                    sav_errno = errno ? errno : ENOMEM;
                }
                ber_bvecfree(value);
            }
        }

        if (reply)
            ldap_msgfree(reply);
        if(*ce_ap)
	    break;
    }

    if (query)
        g_free(query);
    if (ld)
        close_connection(ld);

out:
    if (!*ce_ap) {
        if (!sav_errno) {
            sav_errno = EINVAL;
        }
        SD_setException(&exception, SDStatus_FAILURE, "[BDII][g1_sd_get_ce_ap]: failed", errbuf, errbufsz);
        errno = sav_errno;
        return (-1);
    }
    return (0);
}



/* generates part of bdii filter with accesscontrolbaserule */
char *g1_generate_acbr(const char *glueobject) {

    char tmp[64 + 511];
    char *filter = NULL, *vo = NULL;
    int nb_fqan, filterlen, i;

    if ((vo = vo_global) == NULL)
        return (NULL);

    nb_fqan = fqan_size_global;
    filterlen = nb_fqan * (sizeof (tmp)) + 3 * (64 + 255);

    if ((filter = (char *) calloc(filterlen, sizeof (char))) == NULL){        
        return (NULL);
	}

    snprintf(filter, filterlen,
            "(| (%sAccessControlBaseRule=VO:%s) (%sAccessControlBaseRule=%s) (%sAccessControlRule=%s)",
            glueobject, vo, glueobject, vo, glueobject, vo);

    for (i = 0; i < nb_fqan; ++i) {
        if (fqan_global && fqan_global[i]) {
            snprintf(tmp, sizeof (tmp), " (%sAccessControlBaseRule=VOMS:%s)", glueobject, fqan_global[i]);
            strcat(filter, tmp);
        }
    }

    strcat(filter, ")");
    return (filter);
}

/* get from the BDII the endpoint for the LFC */
int g1_sd_get_lfc_endpoint(char **lfc_endpoint, char *errbuf, int errbufsz) {
    LDAPMessage *entry = NULL;
    char *filter = NULL; 
    char *filter_tmp = NULL;
    int rc = 0;
    LDAPMessage *reply = NULL;
    struct berval **value = NULL;
    int sav_errno = 0;
    *lfc_endpoint = NULL;
    SDException exception;
    LDAP *ld = NULL;
    
     
    if ((filter_tmp = g1_generate_acbr("GlueService")) == NULL){
        SD_setException(&exception, SDStatus_FAILURE, "[BDII][g1_generate_acbr]: Cannot get acbr", errbuf, errbufsz);    
        return (-1);
    }

    rc = asprintf(&filter, QUERY_LFC_ENDPOINT, filter_tmp);
    free(filter_tmp);
    if (rc < 0){
       SD_setException(&exception, SDStatus_FAILURE, "[BDII][g1_sd_get_lfc_endpoint]: Cannot get lfc endpoint", errbuf, errbufsz);    
       return (-1);
     }


    if ((ld = get_connection(&exception, errbuf, errbufsz)) == NULL) {
        SD_setException(&exception, SDStatus_FAILURE, "[BDII][g1_sd_get_lfc_endpoint]: Cannot connect to BDII", errbuf, errbufsz);
        return (-1);
    }    


    while (!*lfc_endpoint && !sav_errno) {

        rc = connect_search_ldap(glue1_base, filter, lfc_endpoint_attrs, ld, &exception, &reply, errbuf, errbufsz);
        if (rc != LDAP_SUCCESS) {
            if (reply) {
                ldap_msgfree(reply);
            }
            sav_errno = errno;
            break;
        }


        if ((entry = ldap_first_entry(ld, reply)) && (value = ldap_get_values_len(ld, entry, ATTR_ENDPOINT))) {
            if (value && value[0]->bv_val) {
                if ((*lfc_endpoint = strdup(value[0]->bv_val)) == NULL)
                    sav_errno = errno ? errno : ENOMEM;
                ldap_value_free_len(value);
            }
        }

        if (reply) {
            ldap_msgfree(reply);
        }       
    }


    if(filter)
    	free(filter);
    if(ld)
    	close_connection(ld);

    if (!*lfc_endpoint) {
        if (!sav_errno) {            
            sav_errno = EINVAL;
        }
        SD_setException(&exception, SDStatus_FAILURE, "[BDII][g1_sd_get_lfc_endpoint]: No LFC Endpoint found", errbuf, errbufsz);
        errno = sav_errno;
        return (-1);
    }

    return 0;
}

int g1_get_voinfo(LDAP *ld, const char *host, const char *spacetokendesc, char **sa_path, char **salocalid,
        char *errbuf, int errbufsz) {

    static char sa_path_atnm[] = "GlueVOInfoPath";
    static char sa_key_atnm[] = "GlueChunkKey";
    static char *template = "(&%s(GlueVOInfoTag=%s)(GlueChunkKey=GlueSEUniqueID=%s))";
    static char *template2 = "(&%s(!(GlueVOInfoTag=*))(GlueChunkKey=GlueSEUniqueID=%s))";
    static char *attrs[] = {sa_key_atnm, sa_path_atnm, NULL};
    int i = 0;
    LDAPMessage *entry = NULL;
    char *filter = NULL, *filter_tmp = NULL; 
    int rc = 0;
    LDAPMessage *reply = NULL;
    struct berval **value = NULL;
    int sav_errno = 0;
    SDException exception;

    if (!host || !sa_path || !salocalid) {
        SD_setException(&exception, SDStatus_FAILURE, "[BDII][g1_get_voinfo]: Invalid Arguments.", errbuf, errbufsz);
        errno = EINVAL;
        return (-1);
    }


    if ((filter_tmp = g1_generate_acbr("GlueVOInfo")) == NULL) {
        SD_setException(&exception, SDStatus_FAILURE, "[BDII][g1_get_voinfo]: Cannot get acbr", errbuf, errbufsz);    
        return (-1);
    }

    if (spacetokendesc)
        rc = asprintf(&filter, template, filter_tmp, spacetokendesc, host);
    else
        rc = asprintf(&filter, template2, filter_tmp, host);

    if (filter_tmp)
        free(filter_tmp);

    if (rc < 0) return (-1);
    *sa_path = *salocalid = NULL;

    while (!*sa_path && !*salocalid && !sav_errno) {
 
        rc = connect_search_ldap(glue1_base, filter, attrs, ld, &exception, &reply, errbuf, errbufsz);
        if (rc != LDAP_SUCCESS) {
            if (reply) {
                ldap_msgfree(reply);
            }
            sav_errno = errno;
            break;
        }


        if ((1 > ldap_count_entries(ld, reply))) {
            if (reply) {
                ldap_msgfree(reply);
            }           
	    sav_errno = EINVAL;
            break;
        }


        if ((entry = ldap_first_entry(ld, reply))) {
            if ((value = ldap_get_values_len(ld, entry, sa_path_atnm)) != NULL) {
                if (value && value[i] && value[i]->bv_val) {
                    if ((*sa_path = strdup(value[i]->bv_val)) == NULL)
                        sav_errno = errno ? errno : ENOMEM;
                    if (value)
                        ldap_value_free_len(value);
                }
            } else if ((value = ldap_get_values_len(ld, entry, sa_key_atnm)) != NULL) {
                if (value && value[i] && value[i]->bv_val) {
                    for (i = 0; value[i]->bv_val && !*salocalid && !rc; ++i) {
                        if (strncmp(value[i]->bv_val, "GlueSALocalID=", 14) == 0)
                            if ((*salocalid = strdup(value[i]->bv_val + 14)) == NULL)
                                sav_errno = errno ? errno : ENOMEM;
                    }
                    if (value)
                        ldap_value_free_len(value);
                }
            } else {
                if (spacetokendesc)
                    SD_setException(&exception, SDStatus_FAILURE, "[BDII][g1_get_voinfo]: GlueVOInfo for tag  and SE wrongly published", errbuf, errbufsz);
                else
                    SD_setException(&exception, SDStatus_FAILURE, "[BDII][g1_get_voinfo]: GlueVOInfo for SE (with no tag) wrongly published", errbuf, errbufsz);
            }

        } else {
            if (spacetokendesc)
                SD_setException(&exception, SDStatus_FAILURE, "[BDII][g1_get_voinfo]: no GlueVOInfo information found about tag  and SE", errbuf, errbufsz);
            else
                SD_setException(&exception, SDStatus_FAILURE, "[BDII][g1_get_voinfo]: no GlueVOInfo information found about SE  (with no tag)", errbuf, errbufsz);
        }
        if (reply) {
            ldap_msgfree(reply);
        }        
    }

    if (filter)
        free(filter); 
 
    if (!*sa_path && !*salocalid) {
        if (!sav_errno) {
            if (spacetokendesc) {
                SD_setException(&exception, SDStatus_FAILURE, "[BDII][g1_get_voinfo]: no GlueVOInfo information found about tag and SE ", errbuf, errbufsz);
            } else {
                SD_setException(&exception, SDStatus_FAILURE, "[BDII][g1_get_voinfo]: no GlueVOInfo information found about SE  (with no tag) ", errbuf, errbufsz);

            }
            sav_errno = EINVAL;
        }	
        errno = sav_errno;
        return (-1);
    }

    return (0);
}

/**
 *  Get from the BDII the SAPath
 *  */

int g1_sd_get_storage_path_local(LDAP *ld, const char *host, const char *salocalid, char **sa_path,
        char **sa_root, char *errbuf, int errbufsz) {

    LDAPMessage *entry = NULL, *reply = NULL;
    char *query = NULL;
    struct berval **value = NULL;
    int sav_errno = 0;
    SDException exception;
    int ret = 0;
    char *vo = NULL;

    if ((vo = vo_global) == NULL){
        SD_setException(&exception, SDStatus_FAILURE, "[BDII][g1_sd_get_storage_path_local]: no VO info ", errbuf, errbufsz);
        return (-1);
    }

    if (salocalid == NULL) {
        salocalid = vo;
    }

    if (!host || !sa_path || !sa_root) {
        errno = EINVAL;
        SD_setException(&exception, SDStatus_FAILURE, "[BDII][g1_sd_get_storage_path_local]: Invalid Arguments.", errbuf, errbufsz);
        return (-1);
    }

    *sa_path = *sa_root = NULL;

    query = g_strdup_printf(QUERY_SA_PATH, salocalid, host);

    while (!*sa_path && !*sa_root && !sav_errno) {
        
        ret = connect_search_ldap(glue1_base, query, sa_path_attrs, ld, &exception, &reply, errbuf, errbufsz);
        if (ret != LDAP_SUCCESS) {
            if (reply) {
                ldap_msgfree(reply);
                reply = NULL;
            }          
            sav_errno = errno;
            break;
        }

        if ((entry = ldap_first_entry(ld, reply))) {
            if ((value = ldap_get_values_len(ld, entry, ATTR_SA_PATH_ATNM)) != NULL) {
                if (value[0]->bv_val && (strncmp(value[0]->bv_val, vo, strlen(vo)) == 0) && (*(value[0]->bv_val + strlen(vo)) == ':')) {
                    if ((*sa_path = g_strdup(value[0]->bv_val + strlen(vo) + 1)) == NULL)
                        sav_errno = errno ? errno : ENOMEM;
                }
                else if ((*sa_path = strdup(value[0]->bv_val)) == NULL) {
                    sav_errno = errno ? errno : ENOMEM;
                }
                if (value)
                    ldap_value_free_len(value);
            }
            else if ((value = ldap_get_values_len(ld, entry, ATTR_SA_ROOT_ATNM))
                    != NULL) {
                if (value[0]->bv_val && (*sa_root = strdup(value[0]->bv_val + strlen(vo) + 1))
                        == NULL) {
                    sav_errno = errno ? errno : ENOMEM;
                    if (value)
                        ldap_value_free_len(value);
                }
            } else {
                SD_setException(&exception, SDStatus_FAILURE, "[BDII][g1_sd_get_storage_path_local]: Both SAPath and SARoot are not set.", errbuf, errbufsz);
                if (reply) {
                    ldap_msgfree(reply);
                }
               
                sav_errno = errno;
                break;
            }
        } else {
            SD_setException(&exception, SDStatus_FAILURE, "[BDII][g1_sd_get_storage_path_local]: No GlueSA information found about VO and SE.", errbuf, errbufsz);
            if (reply) {
                ldap_msgfree(reply);
            }          
            break;
        }
        if (reply) {
            ldap_msgfree(reply);
        }      
    }

    if(query)
    	g_free(query);

    if (!*sa_path && !*sa_root) {
        if (!sav_errno) {            
            sav_errno = EINVAL;
        }
	SD_setException(&exception, SDStatus_FAILURE, "[BDII][g1_sd_get_storage_path_local]: No GlueSA information found about VO and SE.", errbuf, errbufsz);	
        errno = sav_errno;
        return (-1);
    }
    
	
    return (0);
}

int g1_sd_get_storage_path(const char *host, const char *spacetokendesc,
        char **sa_path, char **sa_root, char *errbuf, int errbufsz) {

    char *salocalid = NULL;
    int rc = 0;
    SDException exception;
    LDAP *ld = NULL;
    
 
    if (!host || !sa_path || !sa_root) {
        SD_setException(&exception, SDStatus_FAILURE, "[BDII][g1_sd_get_storage_path]: Invalid Arguments.", errbuf, errbufsz);
        errno = EINVAL;
        return (-1);
    }

   if ((ld = get_connection(&exception, errbuf, errbufsz)) == NULL) {
        SD_setException(&exception, SDStatus_FAILURE, "[BDII][g1_sd_get_storage_path]: Cannot connect to BDII", errbuf, errbufsz);
        return (-1);
    }    
  
    *sa_path = *sa_root = NULL;

    rc = g1_get_voinfo(ld, host, spacetokendesc, sa_path, &salocalid, errbuf, errbufsz);

    if (!*sa_path) {
        rc = g1_sd_get_storage_path_local(ld, host, salocalid, sa_path, sa_root, errbuf, errbufsz);
    }

    if(ld)
    	close_connection(ld);

    return (rc);
}

/**
 *  get from the BDII the SE type ( disk, srm_v1 )
 *
 *  */

int g1_sd_get_se_types_and_endpoints(const char *host, char ***se_types,
        char ***se_endpoints, char *errbuf, int errbufsz) {
    char host_tmp[255];
    int len_tmp = 0;
    char *port = NULL;
    LDAPMessage *entry = NULL;
    SDException exception;
    char *query = NULL;
    int i, nbentries = 0, n = 0, rc = 0;
    char **sep = NULL, **stp = NULL, **st = NULL, **sv = NULL, **ep = NULL;
    struct berval **value = NULL;
    int sav_errno = 0;
    LDAPMessage *reply = NULL;
    LDAP *ld = NULL;

    *se_types = NULL;
    *se_endpoints = NULL;
    
     if(!host)
     {
      SD_setException(&exception, SDStatus_FAILURE, "[BDII][g1_sd_get_se_types_and_endpoints]: Host is empty", errbuf, errbufsz);
      return (-1);
     }
      
        
   if ((ld = get_connection(&exception, errbuf, errbufsz)) == NULL) {
        SD_setException(&exception, SDStatus_FAILURE, "[BDII][g1_sd_get_se_types_and_endpoints]: Cannot connect to BDII", errbuf, errbufsz);
        return (-1);
    }    
 


    strncpy(host_tmp, host, 255);

    if ((port = strchr(host_tmp, ':')) != NULL)
        *port = 0;


    query = g_strdup_printf(QUERY_TYPE_ENDPOINTS, host_tmp, host_tmp, port == NULL ? "" : port + 1);

    while (nbentries < 1 && !sav_errno) {
      
        rc = connect_search_ldap(glue1_base, query, NULL, ld, &exception, &reply, errbuf, errbufsz);
        if (rc != LDAP_SUCCESS) {
            if (reply) {
                ldap_msgfree(reply);
            }            
            rc = -1;
            sav_errno = errno;
            break;
        }

        if ((nbentries = ldap_count_entries(ld, reply)) < 1) {
            SD_setException(&exception, SDStatus_FAILURE, "[BDII][g1_sd_get_se_types_and_endpoints]: No entries for host.", errbuf, errbufsz);
            if (reply) {
                ldap_msgfree(reply);
            }
            rc = -1;            
            break;
        }

        ++nbentries;
        if ((st = calloc(nbentries, sizeof (char *))) == NULL || (sv = calloc(
                nbentries, sizeof (char *))) == NULL || (ep = calloc(nbentries,
                sizeof (char *))) == NULL || (stp = calloc(nbentries,
                sizeof (char *))) == NULL || (sep = calloc(nbentries,
                sizeof (char *))) == NULL) {
            if (reply) {
                ldap_msgfree(reply);
            }            
            sav_errno = errno ? errno : ENOMEM;
            rc = -1;
            break;
        }
        for (entry = ldap_first_entry(ld, reply); entry != NULL && rc == 0; entry
                = ldap_next_entry(ld, entry)) {

            if ((value = ldap_get_values_len(ld, entry, ATTR_ENDPOINT)) != NULL) {
                if (value && value[0]->bv_val) {
                    
                    ep[n] = strdup(value[0]->bv_val);
                    if (value)
                        ber_bvecfree(value);

                    if ((value = ldap_get_values_len(ld, entry, ATTR_TYPE)) == NULL) {
                        if (value && ep[n]) {
                            ber_bvecfree(value);
                            free(ep[n]);
                            continue;
                        }
                    }
                    if (st)
                        st[n] = strdup(value[0]->bv_val);
                    if (value)
                        ber_bvecfree(value);

                    if ((value = ldap_get_values_len(ld, entry, ATTR_VERSION))
                            == NULL) {
                        if (value && ep[n] && st[n]) {
                            ber_bvecfree(value);
                            free(ep[n]);
                            free(st[n]);
                            continue;
                        }
                    }
                    if (sv && value && value[0]->bv_val)
                        sv[n] = strdup(value[0]->bv_val);
                    if (value)
                        ber_bvecfree(value);
                    ++n;
                }
            } else {
                if (port == NULL) {
                    /* If port is not yet defined in host_tmp, and is available
                    // it is copied to host_tmp buffer
                    // ... But it will only be used if there is no GlueService entry */
                    if (value)
                        ber_bvecfree(value);

                    value = ldap_get_values_len(ld, entry, ATTR_SE_TYPE_ATPT);
                    if (value == NULL) {
                        if (nbentries == 1) {
                            sav_errno = EINVAL;
                            SD_setException(&exception, SDStatus_FAILURE, "[BDII][g1_sd_get_se_types_and_endpoints]: No port published in the BDII", errbuf, errbufsz);
                        }
                    } else if (value && value[0]->bv_val && (len_tmp + strlen(value[0]->bv_val) + 1 < 255)) {
                        port = host_tmp + len_tmp;
                        strcpy(port + 1, value[0]->bv_val);
                    } else {
                        if (value)
                            ber_bvecfree(value);
                        sav_errno = ENAMETOOLONG;
                        if (reply) {
                            ldap_msgfree(reply);
                        }                       
                        break;
                    }

                }
                if (value)
                    ber_bvecfree(value);
            }
        }
        if (reply) {
            ldap_msgfree(reply);
        }       
    }



    if (rc < 0) {
        for (i = 0; i < n; ++i) {
            if (st && st[i])
                free(st[i]);
            if (sv && sv[i])
                free(sv[i]);
            if (ep && ep[i])
                free(ep[i]);
        }
        if (st)
            free(st);
        if (sv)
            free(sv);
        if (ep)
            free(ep);
        if (stp)
            free(stp);
        if (sep)
            free(sep);
        errno = sav_errno ? sav_errno : EINVAL;
        SD_setException(&exception, SDStatus_FAILURE, "[BDII][g1_sd_get_se_types_and_endpoints]: No available information", errbuf, errbufsz);
    } else {
        if (n > 0) {
            
            for (i = 0; i < n; ++i) {
                if ((strcmp(st[i], "srm_v1") == 0 || strcmp(st[i], "srm_v2")
                        == 0) && ep[i]) {
                    stp[i] = strdup(st[i]);
                    sep[i] = strdup(ep[i]);
                } else if ((strcasecmp(st[i], "SRM") == 0) && (strncmp(sv[i],
                        "1.", 2)) == 0 && ep[i]) {
                    stp[i] = strdup("srm_v1");
                    sep[i] = strdup(ep[i]);
                } else if ((strcasecmp(st[i], "SRM") == 0) && (strncmp(sv[i],
                        "2.2", 3)) == 0 && ep[i]) {
                    stp[i] = strdup("srm_v2");
                    sep[i] = strdup(ep[i]);
                }
                free(st[i]);
                free(sv[i]);
                free(ep[i]);
            }
            free(st);
            free(sv);
            free(ep);
        } else if (n == 0 && port != NULL && stp && sep) {
            /* There were no GlueService entry...
            // ... so endpoint is hostname:port, and type is disk */
            *port = ':';
            stp[0] = strdup("disk");
            sep[0] = strdup(host_tmp);
        } else {
            free(stp);
            free(sep);
            rc = -1;
            SD_setException(&exception, SDStatus_FAILURE, "[BDII][g1_sd_get_se_types_and_endpoints]: No available information", errbuf, errbufsz);
            errno = EINVAL;

        }

        if (rc == 0) {
            *se_types = stp;
            *se_endpoints = sep;
        }
    }
    
    if(ld)
    	close_connection(ld);
    if(query)
    	g_free(query);

    return (rc);
}

/* get from the BDII the list of supported protocols with their associated
 * port number
 */
int g1_sd_get_seap_info(const char *host, char ***access_protocol, int **port, char *errbuf, int errbufsz) {
    char **ap = NULL, *attr = NULL;
    char *query = NULL;
    struct berval **value = NULL;
    BerElement *ber = NULL;
    int nbentries, i = 0, n = 0, rc = 0;
    int *pn = NULL;
    LDAPMessage *entry = NULL, *reply = NULL;   
    SDException exception;
    int sav_errno = 0;
    LDAP *ld = NULL;

     if(!host)
     {
      SD_setException(&exception, SDStatus_FAILURE, "[BDII][g1_sd_get_seap_info]: Host is empty", errbuf, errbufsz);
      return (-1);
     }


   if ((ld = get_connection(&exception, errbuf, errbufsz)) == NULL) {
        SD_setException(&exception, SDStatus_FAILURE, "[BDII][g1_sd_get_seap_info]: Cannot connect to BDII", errbuf, errbufsz);
        return (-1);
    }    
    
     
    /*prepare the query */
    query = g_strdup_printf(QUERY_PROTO_PORT_TYPE, host);

    while (n < 1 && !sav_errno) {
       
        /*send query to bdii server*/
        rc = connect_search_ldap(glue1_base, query, seap_attrs, ld, &exception, &reply, errbuf, errbufsz);
        if (rc != LDAP_SUCCESS) {
            if (reply) {
                ldap_msgfree(reply);
            }           
            rc = -1;
            sav_errno = errno;
            break;
        }

        /*count entry # */
        if ((nbentries = ldap_count_entries(ld, reply)) < 1) {
            SD_setException(&exception, SDStatus_FAILURE, "[BDII][g1_sd_get_seap_info]: No entries for host.", errbuf, errbufsz);
            if (reply) {
                ldap_msgfree(reply);
            }
            sav_errno = errno ? errno : ENOMEM;           
            rc = -1;
            break;
        }


        nbentries++;
        /*allocate memory */
        if ((ap = (char **) calloc(nbentries, sizeof (char *))) == NULL || (pn
                = (int *) calloc(nbentries, sizeof (int))) == NULL) {
            if (reply) {
                ldap_msgfree(reply);
            }          
            sav_errno = errno ? errno : ENOMEM;
            rc = -1;
            break;
        }

        /* search for the specific values */
        for (entry = ldap_first_entry(ld, reply), n = 0; entry != NULL && rc
                == 0; entry = ldap_next_entry(ld, entry), ++n) {

            for (attr = ldap_first_attribute(ld, entry, &ber); attr != NULL
                    && rc == 0; attr = ldap_next_attribute(ld, entry, ber)) {

                if (!ap[n] && (value = ldap_get_values_len(ld, entry, attr))) {
                    if (value && value[0]->bv_val) {
                        /*do the comparison*/
                        if (strcmp(attr, "GlueSEAccessProtocolType") == 0) {
                            /*save value to ap*/
                            if ((ap[n] = strdup(value[0]->bv_val)) == NULL) {
                                rc = -1;
                                sav_errno = errno ? errno : ENOMEM;
                            }
                        } else {

                            pn[n] = atoi(value[0]->bv_val);
                        }
                        if (value)
                            ber_bvecfree(value);
                    }
                }
                if (attr)
                    ldap_memfree(attr);
            }
            if (ber) {
                ber_free(ber, 0);
                ber = NULL;
            }
        }
        if (reply) {
            ldap_msgfree(reply);
        }        
    }



    /* if something went wrong, give the memory free */
    if (rc < 0) {

        for (i = 0; i < n; ++i) {
            if (ap[i]) {
                free(ap[i]);
            }
        }
        if (ap && pn) {
            free(ap);
            free(pn);
        }
        SD_setException(&exception, SDStatus_FAILURE, "[BDII][g1_sd_get_seap_info]: No available information", errbuf, errbufsz);
        errno = sav_errno ? sav_errno : EINVAL;
    } else /*return the values via pointer*/ {
        *access_protocol = ap;
        *port = pn;
    }
    
    if(query)
    	g_free(query);
    if(ld)
    	close_connection(ld);
		
    return (rc);
}

int g1_sd_is_online(char *host, char *errbuf, int errbufsz) {
    char host_tmp[255];
    LDAPMessage *reply = NULL;   
    SDException exception;
    LDAP *ld = NULL;
    char *query = NULL;
    int rc = 0;
    int is_production;
    
     if(!host)
     {
      SD_setException(&exception, SDStatus_FAILURE, "[BDII][g1_sd_is_online]: Host is empty", errbuf, errbufsz);
      return (-1);
     }

  
    if ((ld = get_connection(&exception, errbuf, errbufsz)) == NULL) {
        SD_setException(&exception, SDStatus_FAILURE, "[BDII][g1_sd_is_online]: Cannot connect to BDII", errbuf, errbufsz);
        return (-1);
    }    
 

    strncpy(host_tmp, host, 255);

    query = g_strdup_printf(QUERY_SE_STATUS, host_tmp);
   
    rc = connect_search_ldap(glue1_base, query, NULL, ld, &exception, &reply, errbuf, errbufsz);
    if (rc != LDAP_SUCCESS) {
        SD_setException(&exception, SDStatus_FAILURE, "[BDII][g1_sd_is_online]: No available information", errbuf, errbufsz);
        goto out;
    }

    if ((ldap_count_entries(ld, reply)) < 1)
        is_production = 0;
    else
        is_production = 1;

   
out:
        if (reply)
            	ldap_msgfree(reply);
	if(query)
		g_free(query);
	if(ld)
		close_connection(ld);

    if (is_production == 0)
        return (-1);
	
   return 0;
}

